@@ -1,10 +1,10 @@
 import pandas as pd
 import numpy as np
-import snowflake.connector
+# import snowflake.connector
 import datetime as dt
 from datetime import timedelta
-import matplotlib.pyplot as plt
-from IPython.display import clear_output
+# import matplotlib.pyplot as plt
+# from IPython.display import clear_output
 import os
 import sys
 from datetime import datetime, timedelta
@@ -16,76 +16,88 @@ sys.path.append("..")
 pd.options.display.float_format = '{:.2f}'.format
 
 
-conn = snowflake.connector.connect(
-                user='SHASHI',
-                password='Wind0ws$',
-                account='zxventureseu.eu-central-1',
-                database='ECOMMERCE',
-                schema='UK_BEER_HAWK')
+# conn = snowflake.connector.connect(
+#                 user='SHASHI',
+#                 password='Wind0ws$',
+#                 account='zxventureseu.eu-central-1',
+#                 database='ECOMMERCE',
+#                 schema='UK_BEER_HAWK')
 
-business = 'BeerHawk'
-store = "uk_co_beerhawk" if business == 'BeerHawk' else         \
-    ('com_emporio' if business == 'Emporio' else
-     ('interdrinks' if business == 'Interdrinks' else
-      ('ar_bevybar' if business == 'CraftSociety' else
-       ('cl_casadelacerveza' if business == 'CasadelaCerveza' else
-        ('mx_beerhouse' if business == 'BeerHouse' else
-         ('uk_atom' if business == 'Atom' else 'courier_ze'))))))
+# business = 'BeerHawk'
+# store = "uk_co_beerhawk" if business == 'BeerHawk' else         \
+#     ('com_emporio' if business == 'Emporio' else
+#      ('interdrinks' if business == 'Interdrinks' else
+#       ('ar_bevybar' if business == 'CraftSociety' else
+#        ('cl_casadelacerveza' if business == 'CasadelaCerveza' else
+#         ('mx_beerhouse' if business == 'BeerHouse' else
+#          ('uk_atom' if business == 'Atom' else 'courier_ze'))))))
 
-raw_path = 'C:/Users/40100204/Desktop/ZX/Core/BeerHawk behavioural clusters/'
+# raw_path = 'C:/Users/40100204/Desktop/ZX/Core/BeerHawk behavioural clusters/'
 
-DATE_ = datetime.now().date() - timedelta(1)
+# DATE_ = datetime.now().date() - timedelta(1)
 
-path = raw_path + 'Datasets/{}_Orders_BS_{}.csv'.format(business, DATE_)
+# path = raw_path + 'Datasets/{}_Orders_BS_{}.csv'.format(business, DATE_)
 
-def raw_orders():
-    """
-    read the transaction data
-    """
-    path = raw_path + 'Datasets/{}_Orders_BS_{}.csv'.format(business, DATE_)
-    if not os.path.isfile(path):
-        raw_orders = pd.read_sql_query("""
-        SELECT ORDER_SPK, ORDER_AK, BUSINESS_SPK, BUSINESS_NAME,
-        ITEM_SKU, ITEM_NAME, PRODUCT_SPK, ORDER_DATE, ORDER_TIME, CUSTOMER_SPK, CURRENCY, UNIT_QUANTITY, 
-        EXTENDED_UNIT_QUANTITY1, EXTENDED_UNIT_QUANTITY2, 
-        UNIT_NET_REVENUE_LOCAL, UNIT_GROSS_REVENUE_LOCAL, UNIT_DISCOUNT_TAX_EXCL_USD,
-        DISCOUNT_NAME, COUPON_CODE, ORDER_STATUS, ORDER_STATUS_TYPE, 
-        ORDER_DAYS_AFTER_FIRST_ORDER, IS_FIRST_ORDER
-        FROM ECOMMERCE.UK_BEER_HAWK.VW_FACT_ORDER_DETAILS
-        WHERE ORDER_DATE>='2020-01-01';
-        """, conn)
+# def raw_orders():
+#     """
+#     read the transaction data
+#     """
+#     path = raw_path + 'Datasets/{}_Orders_BS_{}.csv'.format(business, DATE_)
+#     if not os.path.isfile(path):
+#         raw_orders = pd.read_sql_query("""
+#         SELECT ORDER_SPK, ORDER_AK, BUSINESS_SPK, BUSINESS_NAME,
+#         ITEM_SKU, ITEM_NAME, PRODUCT_SPK, ORDER_DATE, ORDER_TIME, CUSTOMER_SPK, CURRENCY, UNIT_QUANTITY, 
+#         EXTENDED_UNIT_QUANTITY1, EXTENDED_UNIT_QUANTITY2, 
+#         UNIT_NET_REVENUE_LOCAL, UNIT_GROSS_REVENUE_LOCAL, UNIT_DISCOUNT_TAX_EXCL_USD,
+#         DISCOUNT_NAME, COUPON_CODE, ORDER_STATUS, ORDER_STATUS_TYPE, 
+#         ORDER_DAYS_AFTER_FIRST_ORDER, IS_FIRST_ORDER
+#         FROM ECOMMERCE.UK_BEER_HAWK.VW_FACT_ORDER_DETAILS
+#         WHERE ORDER_DATE>='2020-01-01';
+#         """, conn)
 
-        if len(raw_orders) > 0:
-            raw_orders.to_csv(path, index=False)
-    else:
-        raw_orders = pd.read_csv(path)
+#         if len(raw_orders) > 0:
+#             raw_orders.to_csv(path, index=False)
+#     else:
+#         raw_orders = pd.read_csv(path)
 
-    path = raw_path + 'Datasets/{}_Sessions_BS_{}.csv'.format(business, DATE_)
+#     path = raw_path + 'Datasets/{}_Sessions_BS_{}.csv'.format(business, DATE_)
 
-    # print (raw_orders.head())
-    return raw_orders
+#     # print (raw_orders.head())
+#     return raw_orders
 
-def raw_sessions():
-    path = raw_path + 'Datasets/{}_Sessions_BS_{}.csv'.format(business, DATE_)
+# def raw_sessions():
+#     path = raw_path + 'Datasets/{}_Sessions_BS_{}.csv'.format(business, DATE_)
 
-    if not os.path.isfile(path):
-        raw_sessions = pd.read_sql_query("""
-        SELECT ORDER_AK, USERTYPE, "SOURCE", MEDIUM, 
-        KEYWORD, DEVICECATEGORY, REGION, CITY 
-        FROM ECOMMERCE.UK_BEER_HAWK.VW_GA_TRANSACTIONS;
-        """, conn)
+#     if not os.path.isfile(path):
+#         raw_sessions = pd.read_sql_query("""
+#         SELECT ORDER_AK, USERTYPE, "SOURCE", MEDIUM, 
+#         KEYWORD, DEVICECATEGORY, REGION, CITY 
+#         FROM ECOMMERCE.UK_BEER_HAWK.VW_GA_TRANSACTIONS;
+#         """, conn)
 
-        if len(raw_sessions) > 0:
-            raw_sessions.to_csv(path, index=False)
-    else:
-        raw_sessions = pd.read_csv(path)
+#         if len(raw_sessions) > 0:
+#             raw_sessions.to_csv(path, index=False)
+#     else:
+#         raw_sessions = pd.read_csv(path)
 
-    raw_sessions.ORDER_AK = raw_sessions['ORDER_AK'].astype(str)
-    # print(raw_sessions.head())
+#     raw_sessions.ORDER_AK = raw_sessions['ORDER_AK'].astype(str)
+#     # print(raw_sessions.head())
 
-    return raw_sessions
+#     return raw_sessions
 
 def pd_combo_orders(raw_orders, raw_sessions):
+    """Function to combine orders and sessions data for Kegs and machines 
+    and create a grouping of sales for individual customers,
+    which will help in creating customer KPIs
+
+    Args:
+        raw_orders(dataframe): Contains orders at transaction level for each customer
+        raw_session(dataframe): Contains GA sessions data at session level for each customer
+    Returns:
+        pd_df(dataframe): Combined data frame for raw_orders and raw_sessions
+        sales_pd_gb(dataframe): Grouping of sales for each individual customer.
+    
+    """
     raw_df = pd.merge(raw_orders, raw_sessions, on='ORDER_AK', how='inner')
     raw_df['ORDER_DATE'] = pd.to_datetime(raw_df['ORDER_DATE'], format='%Y-%m-%d')
     pd_df = raw_df[raw_df['ITEM_SKU'].str.contains('PD')]
@@ -105,6 +117,18 @@ def pd_combo_orders(raw_orders, raw_sessions):
     return pd_df, sales_pd_gb
 
 def pd_machine_orders(raw_orders, raw_sessions):
+    """Function to combine orders and sessions data for machines 
+    and create a grouping of sales for individual customers,
+    which will help in creating customer KPIs
+
+    Args:
+        raw_orders(dataframe): Contains orders at transaction level for each customer
+        raw_session(dataframe): Contains GA sessions data at session level for each customer
+    Returns:
+        pd_df(dataframe): Combined data frame for raw_orders and raw_sessions
+        sales_pd_gb(dataframe): Grouping of sales for each individual customer.
+    
+    """
     raw_df = pd.merge(raw_orders, raw_sessions, on='ORDER_AK', how='inner')
     raw_df['ORDER_DATE'] = pd.to_datetime(raw_df['ORDER_DATE'], format='%Y-%m-%d')
     pd_df = raw_df.loc[~(np.isnan(raw_df.EXTENDED_UNIT_QUANTITY1)) & (raw_df['EXTENDED_UNIT_QUANTITY1'] != 0)]
@@ -124,6 +148,18 @@ def pd_machine_orders(raw_orders, raw_sessions):
     return pd_df, sales_pd_gb
 
 def pd_keg_orders(raw_orders, raw_sessions):
+    """Function to combine orders and sessions data for Kegs 
+    and create a grouping of sales for individual customers,
+    which will help in creating customer KPIs
+
+    Args:
+        raw_orders(dataframe): Contains orders at transaction level for each customer
+        raw_session(dataframe): Contains GA sessions data at session level for each customer
+    Returns:
+        pd_df(dataframe): Combined data frame for raw_orders and raw_sessions
+        sales_pd_gb(dataframe): Grouping of sales for each individual customer.
+        
+    """
     raw_df = pd.merge(raw_orders, raw_sessions, on='ORDER_AK', how='inner')
     raw_df['ORDER_DATE'] = pd.to_datetime(raw_df['ORDER_DATE'], format='%Y-%m-%d')
     pd_df = raw_df.loc[~(np.isnan(raw_df.EXTENDED_UNIT_QUANTITY2)) & (raw_df['EXTENDED_UNIT_QUANTITY2'] != 0)]
@@ -143,6 +179,20 @@ def pd_keg_orders(raw_orders, raw_sessions):
     return pd_df, sales_pd_gb
 
 def filter_data(pd_df, business, date_filter, region_list, order_status_type, order_status):
+    """Function to create filters on the orders based on 
+    business, date, region, order_status_type and order_status
+
+    Args:
+        pd_df(dataframe): Contains orders at transaction level for each customer
+        business(list): 'BeerHawk'
+        date_filter(date): The date from which data should be considered for clustering
+        region_list(list): limit to orders from following region 
+        order_status_type: 'Valid'
+        order_status(list): ['complete','serviceissueresolved']
+    Returns:
+        pd_df(dataframe): Filtered data frame for raw_orders and raw_sessions
+        
+    """
     # Apply filters to sales DF
     print('Total number of Orders')
     print(len(pd_df))
@@ -170,6 +220,15 @@ def filter_data(pd_df, business, date_filter, region_list, order_status_type, or
     return pd_df
 
 def create_customer_kpis(pd_df, sales_pd_gb):
+    """Function to create customer KPIs (e.g. AOV, AOF etc)
+
+    Args:
+        pd_df(dataframe): Combined dataframe for raw_orders and raw_sessions
+        sales_pd_gb(dataframe): Grouped sales for each individual customer.
+    Returns:
+        customer_df(dataframe): KPIs(e.g. AOV, AOF etc) for each individual customer.
+        
+    """
     # Calculate average unique skus per order
     unique_sku_per_order = pd_df.groupby(['CUSTOMER_SPK', 'ORDER_SPK'])['PRODUCT_SPK'].nunique().reset_index()
     average_unique_skus = unique_sku_per_order.groupby('CUSTOMER_SPK')['PRODUCT_SPK'].mean().reset_index()
@@ -185,6 +244,14 @@ def create_customer_kpis(pd_df, sales_pd_gb):
     customer_df['average_order_frequency'] = ((sales_pd_gb['ORDER_DATE']['max'] - sales_pd_gb['ORDER_DATE']['min']) / timedelta(1)) / \
                                              sales_pd_gb['ORDER_SPK']['nunique']
 
+    # Calculate total number of orders
+    customer_df['total_number_of_orders'] = sales_pd_gb['ORDER_SPK']['nunique'] 
+
+    # # Calculate average number of orders per customer
+    # customer_df['average_number_of_orders_per_customer'] = sales_pd_gb['ORDER_SPK']['nunique'] / \
+    #                                           ((sales_pd_gb['ORDER_DATE']['max'] - sales_pd_gb['ORDER_DATE']['min']) / timedelta(1))
+    # customer_df[customer_df['average_number_of_orders_per_customer'] == np.inf] = 0.001                                             
+
     # Calculate average order value
     customer_df['average_order_value'] = sales_pd_gb['UNIT_GROSS_REVENUE_LOCAL']['sum'] / \
                                          sales_pd_gb['ORDER_SPK']['nunique']
@@ -213,11 +280,22 @@ def create_customer_kpis(pd_df, sales_pd_gb):
     return customer_df
 
 def browsing_category(pd_df, medium_list, source_list):
+    """Function to create a browsing behaviour dataframe which
+    consists of only top mediums and sources.
+
+    Args:
+        pd_df(dataframe): Combined dataframe for raw_orders and raw_sessions
+        medium_list(list): Limit medium to top categories
+        source_list(list): Limit source to top categories
+    Returns:
+        browsing_behaviour_df(dataframe): Combined dataframe of Source, Medium, Device
+        customer_total_page_views_gb(dataframe): Grouped browsing_behaviour_df on individual customers
+    """
     browsing_behaviour_df = pd_df[['CUSTOMER_SPK', 'SOURCE', 'MEDIUM', 'DEVICECATEGORY']]
     browsing_behaviour_df['MEDIUM'] = browsing_behaviour_df['MEDIUM'].replace(['Nosto'], 'nosto')
     browsing_behaviour_df['MEDIUM'] = browsing_behaviour_df['MEDIUM'].replace(['affiliation'], 'affiliate')
     browsing_behaviour_df['MEDIUM'] = browsing_behaviour_df['MEDIUM'].replace(['paid_pd', 'Paid', 'Paid_PD'], 'paid')
-    browsing_behaviour_df['MEDIUM'] = browsing_behaviour_df['MEDIUM'].replace(['Organic_Social', 'organic_social', 'organic social'], 'Organic_Social')
+    browsing_behaviour_df['MEDIUM'] = browsing_behaviour_df['MEDIUM'].replace(['Organic_Social', 'organic_social', 'organic social'], 'organic_social_medium')
 
     browsing_behaviour_df['SOURCE'] = browsing_behaviour_df['SOURCE'].replace(['beer_hawk', 'Beer Hawk', 'beer+hawk', 'beerhawk'], 'beer hawk')
     browsing_behaviour_df['SOURCE'] = browsing_behaviour_df['SOURCE'].replace(['facebook', 'm.facebook.com', 'l.facebook.com', 'facebook.com', 'lm.facebook.com', 'Facebook'], 'facebook')
@@ -236,6 +314,17 @@ def browsing_category(pd_df, medium_list, source_list):
     return browsing_behaviour_df, customer_total_page_views_gb
 
 def clean_behaviour_measure(browsing_behaviour_df, customer_total_page_views_gb, browsing_variable):
+    """Function to create a browsing behaviour measure for each customers.
+
+    Args:
+        browsing_behaviour_df(dataframe): Combined dataframe of Source, Medium, Device
+        customer_total_page_views_gb(dataframe): Grouped browsing_behaviour_df on individual customers
+        browsing_variable : DEVICECATEGORY, MEDIUMCATEGORY, SOURCECATEGORY
+    Returns:
+        browsing_measure_gb: Affinity scores of each customer towards different browsing medium
+                            (which could be Device, Medium and Sources)
+    """
+
     browsing_measure_gb = browsing_behaviour_df.groupby(['CUSTOMER_SPK', browsing_variable]).size().reset_index()
     browsing_measure_gb = browsing_measure_gb.rename(columns={0: 'PAGEVIEWS'})
     # Merge total page views into gb
@@ -254,6 +343,18 @@ def clean_behaviour_measure(browsing_behaviour_df, customer_total_page_views_gb,
     return browsing_measure_gb
 
 def browsing_pivot(browsing_behaviour_df, customer_total_page_views_gb):
+    """Function to create a pivot table showcasing the affinity of customers
+        towards each browsing medium (which could be Device, Medium and Sources)
+
+    Args:
+        browsing_behaviour_df(dataframe): Combined dataframe of Source, Medium, Device
+        customer_total_page_views_gb(dataframe): Grouped browsing_behaviour_df on individual customers
+        
+    Returns:
+        browsing_pivot: Pivot table showing affinity scores of each customer towards different browsing medium
+                            (which could be Device, Medium and Sources)
+    """
+
     # Create browsing behaviour outputs
     device_gb = clean_behaviour_measure(browsing_behaviour_df, customer_total_page_views_gb, 'DEVICECATEGORY')
     medium_gb = clean_behaviour_measure(browsing_behaviour_df, customer_total_page_views_gb, 'MEDIUMCATEGORY')
@@ -268,6 +369,18 @@ def browsing_pivot(browsing_behaviour_df, customer_total_page_views_gb):
     return browsing_pivot
 
 def algo_data(customer_df, browsing_pivot):
+    """Function to create input filed to be used by clustering algorithms.
+
+    Args:
+        customer_df(dataframe): KPIs(e.g. AOV, AOF etc) for each individual customer.
+        browsing_pivot(dataframe): Pivot table showing affinity scores of each customer 
+                                    towards different browsing medium
+                                    (which could be Device, Medium and Sources)
+        
+    Returns:
+        input_df(dataframe): Combined input data to be fed to the clustering algo
+        data_scaled(dataframe): Input data scaled and normalized
+    """
     # Identify customers that are within all the measures
     customer_list = list(customer_df['CUSTOMER_SPK'].unique())
     browsing_list = list(browsing_pivot.CUSTOMER_SPK.unique())
@@ -288,18 +401,28 @@ def algo_data(customer_df, browsing_pivot):
     # print(data_scaled.head())
     return input_df, data_scaled
 
-
 def generate_cluster_inputs(initial_cluster_df):
-    # Create cluster GB
+    """Function to create output from clustering ready to be used by PBI reports.
+
+    Args:
+        initial_cluster_df(dataframe): Clustering results generated from Agglomerative clustering
+        
+    Returns:
+        individual_cluster_gb(dataframe): Clustering results to be used by PBI reports 
+        
+    """
     individual_cluster_gb = initial_cluster_df.groupby('Cluster').agg({
         'CUSTOMER_SPK': pd.Series.nunique,
+        'total_number_of_orders': ['mean', 'std'],
         'average_order_frequency': ['mean', 'std'],
+        'average_number_of_orders_per_customer': ['mean', 'std'],
         'average_order_value': ['mean', 'std'],
         'average_items_per_order': ['mean', 'std'],
         'average_price_per_item': ['mean', 'std'],
         'average_unique_skus_per_order': ['mean', 'std'],
         'average_discount_level': ['mean', 'std'],
         'flash_sale_percentage': ['mean', 'std'],
+        # Limit source to top categories
         'google': ['mean', 'std'],
         'beer hawk': ['mean', 'std'],
         '(direct)': ['mean', 'std'],
@@ -308,6 +431,7 @@ def generate_cluster_inputs(initial_cluster_df):
         'facebook': ['mean', 'std'],
         #         'perfectdraft.com': ['mean', 'std'],
         'organic_social': ['mean', 'std'],
+        # Limit Medium to top categories
         'organic': ['mean', 'std'],
         'email': ['mean', 'std'],
         '(none)': ['mean', 'std'],
@@ -317,7 +441,7 @@ def generate_cluster_inputs(initial_cluster_df):
         'paidsocial': ['mean', 'std'],
         # 'nosto': ['mean', 'std'],
         'paid': ['mean', 'std'],
-        'Organic_Social': ['mean', 'std'],
+        'organic_social_medium': ['mean', 'std'],
         'desktop': ['mean', 'std'],
         'mobile': ['mean', 'std'],
         'tablet': ['mean', 'std']})
@@ -326,5 +450,4 @@ def generate_cluster_inputs(initial_cluster_df):
     individual_cluster_gb = individual_cluster_gb.reset_index(drop=True)
     individual_cluster_gb.columns = individual_cluster_gb.columns.map('-'.join).str.strip('-')
 
-    return individual_cluster_gb
-
+    return individual_cluster_gb
\ No newline at end of file
